#
# kernel-CVD-fishing.R,  1 Sep 24
#
# Data from:
# CVEs
# https://github.com/nluedtke/linux_kernel_cves
# Kernel release dates
# Wikipedia

library("colorspace")
library("plyr")
library("survival")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(2)

# Strip rows with illformed version numbers
clean_ver_str=function()
{
t=subset(lcve, !grepl("[a-z]$", lcve$start_version))
t=subset(t, !grepl("[a-z]$", t$end_version))

return(t)
}


lcve=read.csv("linux_CVEs.csv", sep=" ")

lcve$start_version=sub("^v", "", lcve$start_version)
lcve$start_version=sub("-rc[1-9]$", "", lcve$start_version)
lcve$end_version=sub("^v", "", lcve$end_version)
lcve$end_version=sub("-rc[1-9]$", "", lcve$end_version)

lcve=clean_ver_str()

nrow(lcve)


kr=read.csv("kernel-releases.csv")

kr$release_date=as.Date(kr$Original.release.date, format="%d %B %Y")
kr$EOL_date=as.Date(paste0("15 ", kr$EOL), format="%d %B %Y")

unq_ver=unique(kr$Version)

lcve$release_date=as.numeric(mapvalues(lcve$start_version, unq_ver, kr$release_date))
lcve$EOL_date=as.numeric(mapvalues(lcve$start_version, unq_ver, kr$EOL_date))
lcve$fault_date=as.numeric(mapvalues(lcve$end_version, unq_ver, kr$release_date))

# Survival of all cleaned CVEs
lcve_surv=Surv(lcve$fault_date-lcve$release_date,
			event=(lcve$fault_date <= 100000), type="right")

# How many discovered before EOL?
# lcve_surv=Surv(lcve$fault_date-lcve$release_date,
# 			event=(lcve$fault_date <= lcve$EOL_date), type="right")

lcve_mod=survfit(lcve_surv ~ 1)
plot(lcve_mod, col=pal_col[c(1, 2, 2)], conf.int=TRUE,
	xaxs="i", yaxs="i",
	ylim=c(0, 1),
	xlab="Days", ylab="Survival")

dev.copy(dev=png, file="Linux-CVE-survival.png")
dev.off()

